import urllib2,urllib, json
import requests
import StringIO
import Config
from threading import Thread
import InMemoryZipFile
import time
import joblist


class BiodappsConnector:
    """
    creates a connection to BioDApps to get completed jobs
    this is the Super Class to the Pirates and The Maestro Connector
    """
     
    def __init__(self, dataprovider = None, callback = None):
        """
        Constructor 
        sets all important variables also from subclasses
        @param dataprovider: is the date which will send
        @param callback: callback for updating progress
        """
        self.dataprovider = dataprovider
        self.callback = callback
        self.config = Config.Config.getInstance()
        self.url = self.config.getUrl()
        self.jobId = None
        self.filename = None
        
    def setJobId(self, jobId):
        """
        @param jobId: sets jobId
        """
        self.jobId = jobId
    
    def _checkConektion(self):
        """
        Checks the Connection to BioDApps
        """
        response = urllib2.urlopen(self.url % "").read()
        return (response == "Hey! I knew it! It works! AWESOME!!!")
    
    def _getJobId(self, service):  
        """
        Gets the JobId 
        @param service: stands for pirates or maestro
        """ 
        if self.jobId == None:
            urlJob = self.url % "getJobId"
            data = {"service":service}
            data = json.dumps(data)
            req = urllib2.Request(urlJob, data, {'Content-Type': 'application/json'})
            response = urllib2.urlopen(req)
            result = json.load(response)
            self.jobId = result["jobId"]
    
    def _upAndDownLoadData(self, data, command):
        """
        @param data: is the data you want to up- or download
        @param command: runService or checkJob
        
        """
        req = urllib2.Request(self.url % command, data, {'Content-Type': 'application/json'})
        response = urllib2.urlopen(req)
        result = json.load(response)
        return result
    
    def _uploadZip(self):
        """
        Uploads all pdbs for the Job as an zip File
        """
        #pdb files
        pdbfiles = self.dataprovider.getPdbs()
    
        #open zip file in memory
        zipFo = InMemoryZipFile.InMemoryZipFile()
        
        #add pdb strings to zip file
        for pdb in pdbfiles:
            zipFo.write("%s.pdb" % (pdb[0]), pdb[1])
            
        #upload zip file  
        response = zipFo.uploadFile(self.url % "upload")
        filenames = eval(response)["filenames"]
    
        #move uploaded file to job folder
        param = [{'files':filenames, 'command':'upload', 'names':['piratesUpload.zip']}]
        data = {"service":"Pirates", 
                "jobId": self.jobId, 
                "param": param}
    
        data = json.dumps(data)
        
        self._upAndDownLoadData(data, "runService")
        
    def _checkJobStatus(self):
        """
        Checks the Job status 
        @return: if it is still running or not and for Maestro the result with the Progress
        """
        data = {"jobId" : self.jobId}
        data = urllib.urlencode(data)
        requrl = self.url % "checkJobStatus" + "?" + data
        print "@checkJobStatus", requrl
        #- request status
        req = urllib2.Request(requrl)
        response = urllib2.urlopen(req)
        result = json.load(response)
        print "@checkJobStatus2", result
        fin = result["status"]         
        return (fin == "running", result)
        
    def _checkJobStatusWithThread(self, t ):
        """
        @param t: is a Thread 
        check the JobStatus and whether t is alive   
        """
        return self._checkJobStatus()[0] or t.is_alive()
            
class PiratesConnector(BiodappsConnector):
    """
    creates a connection to BioDApps to send a PiratesJobs
    extends BiodappsConnector
    """
    
    def __init__(self, dataprovider = None, callback = None):
        """
        Constructor
        Superclass init is called
        
        @param dataprovider: is the date which will send
        @param callback: callback for updating progress
        """
        BiodappsConnector.__init__(self, dataprovider, callback)
           
    def _sendJobPirates(self): 
        """
        send all data the Pirates job needs (sends dataprovider)
        """        
        self.dataprovider.setJobId(self.jobId)
    
        data = json.dumps(self.dataprovider.getJson())#getJosn
        self._upAndDownLoadData(data, "runService")
            
    def getAdapterStatus(self):
        """
        gets the status of the different MStA Method 
        """
        param = [{'command': 'adapterStatus'}]
        data = {"service":"Pirates", 
            "jobId": self.jobId, 
            "param": param}
        data = json.dumps(data)
        return self._upAndDownLoadData(data, "runService")
            
    def piratesGetFasta(self, jobId = None, alignmentMethod = "consensusAlignment"):
        """
        get and return the result of a job as a Fasta for Pirates 
        @param jobId: define for which jobId get the Fasta
        @param alignmentMethod: define for which MStA Method get the Fasta
        """
        if (jobId != None):
            self.jobId = jobId
        path = "download/Pirates?src=/home/share/biodapps/jobs/%s/%s.fasta" % (self.jobId, alignmentMethod)
        requrl = self.url % path
        req = urllib2.Request(requrl)
        response = urllib2.urlopen(req)
        result = [response, requrl]
        return result#.read()
    
    def piratesGetXml(self, jobId = None, alignmentMethod = "consensusAlignment"):
        """
        get and return the result of a job as an XML for Pirates
        @param jobId: define for which jobId get the XML
        @param alignmentMethod: define for which MStA Method get the XML 
        """
        if (jobId != None):
            self.jobId = jobId
        path = "download/Pirates?src=/home/share/biodapps/jobs/%s/%s.xml" % (self.jobId, alignmentMethod)
        requrl = self.url % path
        req = urllib2.Request(requrl)
        response = urllib2.urlopen(req)
        return response.read()
    
    def getUsedAdapter(self, jobId = None):
        """
        get and return the UsedAdapter of an PiratesJob
        @param jobId: define for which jobId get the UsedAdapter
        """
        if (jobId != None):
            self.jobId = jobId
        param = [{'command': 'usedAdapter'}]
        data = {"service":"Pirates", 
                "jobId": self.jobId, 
                "param": param}

        data = json.dumps(data)
        req = urllib2.Request(self.url % "runService", data, {'Content-Type': 'application/json'})
        response = urllib2.urlopen(req)
        result = json.load(response)
        return result

    def getResult(self, jobId = None, alignmentMethod = "consensusAlignment"):
        """
        get and return the result of a job as a result for Pirates 
        The Result is a List of Fasta, XML and UsedAdapter
        @param jobId: define for which jobId get the Result
        @param alignmentMethod: define for which MStA Method get the Result
        """
        if (jobId != None):
            self.jobId = jobId
        result = []
        result.append(self.piratesGetFasta(jobId, alignmentMethod))
        result.append(self.piratesGetXml(jobId, alignmentMethod))
        result.append(self.getUsedAdapter(jobId))
        return result
        
    def _checkJobStatusWithThread(self, t ):
        """
        Override the _checkJobStatusWithThread from BiodappsConnector
        to use the callback so the Pirates Result Gui knows theAdapterStatus
        """
        if self.callback != None:
            try:
                self.callback(self.getAdapterStatus()['result'])
            except:
                self.callback = None
        return self._checkJobStatus()[0] or t.is_alive()

    def startPiratesJob(self):
        """
        starts a Pirates Job and uses all functions called before
        returns a set at the first index the Fasta, at the second index the XML and the third index the UsedAdapter
        use the callback so the Pirates Result Gui knows the Status
        """

        if self._checkConektion():
            print "-----------------------------------start----------------------------------------------"
            if self.callback != None:
                try:
                    self.callback({"progress": "Starting"})
                except:
                    self.callback = None
                
            self._getJobId("Pirates")
            
            print "-----------------------------------jobid----------------------------------------------"
            if self.callback != None:
                try:
                    self.callback({"progress": "JobID received"})
                except:
                    self.callback = None
                    
                    
            jl = joblist.JobList.getInstance()
            jl.saveFile(self.jobId, "Pirates")
            print "-----------------------------------write file----------------------------------------------"
                 
            if self.callback != None:
                try:
                    self.callback({"progress": "Upload Data"})
                except:
                    self.callback = None
                    
            self._uploadZip()
            print "-----------------------------------upload data----------------------------------------------"
            if self.callback != None:
                try:
                    self.callback({"progress": "Data Uploaded"})
                except:
                    self.callback = None
                    
            t  = Thread(target = self._sendJobPirates)
            t.start()
            print "-----------------------------------start job----------------------------------------------"

            time.sleep(2)
            while self._checkJobStatusWithThread(t):
                print "-----------------------------------wait----------------------------------------------"
                time.sleep(5)

            print "-----------------------------------finish----------------------------------------------"
            if self.callback != None:
                try:
                    self.callback({"progress": "Finished"})
                except:
                    self.callback = None
            result = self.getResult()
            return result
        else :
            print "NO SERVER CONNECTION"
 
class MaestroConnector(BiodappsConnector):
    """
    creates a connection to BioDApps to send a MaestroJob
    extends BiodappsConnector
    """
    def __init__(self, dataprovider = None, callback = None):
        """
        Constructor
        Superclass init is called
        
        @param dataprovider: is the date which will send
        @param callback: callback for updating progress
        """
        BiodappsConnector.__init__(self, dataprovider, callback)
    
    def uploadMaestro(self): 
        """
        Uploads the pdb for the maestro Job
        """  
        #get structure
        pdbstr = self.dataprovider.getPdbs()
    
        #- open file in memory and add pdb string
        strfi = StringIO.StringIO(pdbstr[1])
    
        #- upload structure
        response = requests.post(self.url % "upload", files={'upload': strfi})
        filename = eval(response.text)["filenames"]
        
        self.filename = filename[0]
    
        #- move file to job folder (server)
        data = {"service" : "Mollib", 
            "jobId" : self.jobId, 
            "param" : [{'pdbFile': filename[0], 'orig_pdbFile' : pdbstr[0], 'command' : 'upload'}]};
    
        data = json.dumps(data)        
        self._upAndDownLoadData(data, "runService")
        
    def sendJobMeastro(self): 
        """
        send all data the Maestro job needs (sends dataprovider)
        """        
        self.dataprovider.setJobId(self.jobId)
        self.dataprovider.setFilename(self.filename)
        
        data = json.dumps(self.dataprovider.getJson())
        self._upAndDownLoadData(data, "runService")
        
    def getResults(self,jobId = None):
        """
        get and returns the result of an job as an json for Maestro
        """
        if (jobId != None):
            self.jobId = jobId
        data = {"jobId" : self.jobId};
        data = json.dumps(data)
        result = self._upAndDownLoadData(data, "checkJob")
        return result
        
    def startMaestroJob(self, uploadZip = False):
        """
        starts an Maestro Job and use all functions called before
        returns the result as json
        use the callback so the Maestro ProgressBar knows the Status
        @param uploadZip: define if it is needed to uploaded 
        """
        if self._checkConektion():
            self._getJobId("Maestro")
            
            service = "Maestro"
            
            if self.dataprovider != None:
                if 'param' in self.dataprovider.data.keys():
                    if self.dataprovider.data["param"][0]["command"] == "scoreStructure":
                        service = "MaestroS"
            
            jl = joblist.JobList.getInstance()
            jl.saveFile(self.jobId, service)
            
            if uploadZip == True:
                self._uploadZip()
            else:
                self.uploadMaestro()
                
            t  = Thread(target = self.sendJobMeastro)
            t.start()
            
            time.sleep(2)
            status = self._checkJobStatusWithThread(t)
            while status[0]:
                
                status = self._checkJobStatusWithThread(t)
                
                # care for progressbar
                prog = float(status[1]['progress'])
                if self.callback != None:
                    try:
                        self.callback(prog)
                    except:
                        self.callback = None
                
                time.sleep(5)
                
            return self.getResults()
        else :
            print "NO SERVER CONNECTION" 
    
    def _checkJobStatusWithThread(self, t ):
        """
        Override the _checkJobStatusWithThread from BiodappsConnector
        to return the complete Status from _checkJobStatus
        """
        status = self._checkJobStatus()
        return (status[0] or t.is_alive(), status[1])  
        