import string
import chimera
from chimera import runCommand
import sys
from threading import Thread
import xml.dom.minidom
from chimera import selection


def validatePirates(PiratesGUI):
    '''
    validates input of PiratesGUI before data will be extracted
    when not valid: prints Error-Message to IDLE and returns FALSE
    when get's through all tests without detecting an error return TRUE at the end
    validates if:
        has user loaded at least one PDBS? (has to be done before next setp can be done)
        has user selected at least tow pdbs?
        has user selected at least one of the methods?        
        
    @param PiratesGUI: gets whole GUI where data was inserted by user, so it can extract the values
    '''
    #is there anything selected (would case crash in selection_get())
    if PiratesGUI.pdbIdMenu.curselection() == ():
        return False
    # Select/Structure: min 2 selected 
    PiratesGUI.selId = PiratesGUI.pdbIdMenu.selection_get()
    IDList = str(PiratesGUI.selId).split()
    if(len(IDList)<2):
        return False
    
    # MStA Methos: min 1 selected 
    sel = [PiratesGUI.var1.get(), PiratesGUI.var2.get(), PiratesGUI.var3.get(), PiratesGUI.var4.get(), PiratesGUI.var5.get(), PiratesGUI.var6.get(), PiratesGUI.var7.get(), PiratesGUI.var8.get()]
    one = False
    for i in range(0, 7):
        if (sel[i]==1):
            one = True
    if(one != True):
        return False
    else:
        return True

class dataProviderPirates:
    '''
    shall be used in apply, after validation
    collects data which shall be sent to BioDApps
    also needs job ID 
    '''
    def __init__(self):
        '''
        param will be added to data later
        Some params have fix values
        data is map which contains everything except of pdbs
        param is list of maps
        pdbs is List 
        later data and pdbs will be sent to BioDApps Server 
        '''
        self.param = [{"selectedAdapters": [], "command": "create"}]
        self.pdbs = []
        self.data = {}
        self.data["service"] = "Pirates"
    
    def readGUI(self, PiratesGUI):
        '''
        extracts all needed data from GUI
        self.param[0]["selectedAdapters"] -> is list which contains all methods to be done#
        In GUI var1 to var8 stand for the methods, if their value (get it with .get()) is 1 then user has selected this method 
        no data left to extract from GUI (only JOBId and PDBIds which will be added later is extra functions)
        
        @param PiratesGUI: gets GUI itself to extract data 
        '''
        #called in apply
        #self.var# presents methods, self.var1.get() returns 1(selected) or 0(not selected)
        #fill list with 0/1 in right order
        sel = [PiratesGUI.var1.get(), PiratesGUI.var2.get(), PiratesGUI.var3.get(), 
               PiratesGUI.var4.get(), PiratesGUI.var5.get(), PiratesGUI.var6.get(), 
               PiratesGUI.var7.get(), PiratesGUI.var8.get()]
        
        if (sel[0]==1):
            self.param[0]["selectedAdapters"].append("mass")
        if (sel[1]==1):
            self.param[0]["selectedAdapters"].append("matt")
        if (sel[2]==1):
            self.param[0]["selectedAdapters"].append("mustang")
        if (sel[3]==1):
            self.param[0]["selectedAdapters"].append("mistral")
        if (sel[4]==1):
            self.param[0]["selectedAdapters"].append("mapsci")
        if (sel[5]==1):
            self.param[0]["selectedAdapters"].append("threedcomb")
        if (sel[6]==1):
            self.param[0]["selectedAdapters"].append("smolign")
        if (sel[7]==1):
            self.param[0]["selectedAdapters"].append("mstali")
    
    #pdbList: [(PDBID, str(PDB)),...]
    def setPdbs(self, pdbList):
        '''
        pdbList contains the pdbs which shall be sent to Server
        @param pdbList: List with all pdbs which shall be sent to Server
        ''' 
        for pdb in pdbList:
            self.pdbs.append(pdb)
    
    def getPdbs(self):
        '''
        returns list with tuples
        '''
        return self.pdbs
    
    def setJobId(self,JobId):
        '''
        saves JobId in data construct which will be sent to BioDApps
        @param JobId: retrieved from Server 
        '''
        self.data["jobId"]=JobId
                
    def getJobId(self):
        '''
        if JobId has already been set it will be returned
        otherwise -1 will be returned
        '''
        if(self.data.has_key("jobId")):
            return self.data["jobId"]
        else:
            return -1
    
    def getJson(self):
        '''
        returns whole data construct which will be send to BioDApps Server
        Won't be changed anymore before sanding
        has to put the list 'param' into 'data' (data will be returned)
        '''
        self.data["param"] = self.param
        return self.data
 
def getCoreAlignment(xmlresult, structures):
    """
    @param xmlresult: the XML result contains the matches 
    @param structures: the structures which are loaded 
    
    parse the XML to get which residues from which PDB stay together
    the structures are needed to know which PDB has which ID
    the core Alignment is a list of Strings
    a String consists of the model.id and all CAlpha if there is more then 1 CAlpha it takes the first
    also skips the Gapes
    @return: the core Alignment 
    """
    dom = xml.dom.minidom.parseString(xmlresult)
    matches = []
    
    pdbs = dom.getElementsByTagName("member")
    for pdb in pdbs:
        matches.append("#" + str(structures[pdb.childNodes[0].nodeValue.split(".")[0]]['id']) + ":")# "#" + pdb.childNodes[0].nodeValue.split(".")[0] + ":"
  
    rows = dom.getElementsByTagName("row")
    
    for row in rows:
        gapeMatch = []
        i = 0
        isMeq = 0
        for meq in row.childNodes:
            
            if isMeq % 2 == 1:
                meqv = meq.childNodes[0].nodeValue.split(":")
                
                if  meqv[-1].strip() == "":
                    gapeMatch.append(meqv[0].strip())
                else:
                    gapeMatch.append(meqv[0].strip() + "." + meqv[-1].strip())               
            
            isMeq += 1

        if not "---------.---------" in gapeMatch:
            for meq in gapeMatch: 
                sel = matches[i].split(":")[0] + ":" + meq + "@CA"  
                select = selection.OSLSelection(sel) 
                atom = select.atoms()
                ca = atom[0].altLoc
                
                matches[i] += meq + "@CA"
                if ca != "":
                    matches[i] += "." + ca
                matches[i] += ":"
                
                i += 1

    i = 0
    for match in matches:
        match = match[:-1]
        matches[i] = str(match)
        i += 1
        
    return matches   

def selectCoreAlignment(matches):
    """
    @param matches: get the core Alignment
    use the Chimera command select to Select the core Alignment
    """
    s = "select "
    for i in matches:
        s += i + " "
    s = s[:-1]
    s = s.replace("@CA", "")
    try:
        print s
        runCommand(s)
    except Exception as e:
        print "error"
        print e

def superPos(matches):  
    """
    @param matches: get the core Alignment
    superimpose the structures in Chimera
    use the Chimera command match to superimpose
    """  
    for i in [1, len(matches)-1]:
        s = "match " + matches[i] + " " + matches[0]
        try:
            runCommand(s)
        except Exception as e:
            print "error"
            print e
        
    