import StringIO
from Tkinter import *  # @UnusedWildImport
import tkMessageBox
from chimera.baseDialog import ModelessDialog
from threading import Thread
import chimera
from __init__ import validatePirates, dataProviderPirates, superPos, getCoreAlignment, selectCoreAlignment
from BioDApps_Core import Connector

class PiratesDialog(ModelessDialog):
    '''
    Generates and fills GUI for the input of the user to send a PiratesJob
    You can choose in a List of the loaded molecules which should be compared
    You can send a Job via the BiodAppsConnector to the Pirates Server to get the result
    
    inherited from ModelessDialog  
    '''
    title="Pirates"
    name="Pirates"
    buttons=('Close', 'Apply')
    
    def __init__(self, **kw):
        """
        Constructor
        instantiating the GUI
        
        Special:
        addRemoveHandler/addAddHandler (Observer to get notified if there is a new molecule
        """
        
        self.v = StringVar()
        self.pdbIdMenu = None
        ModelessDialog.__init__(self, **kw)
        
        self._closeModelHandlerID = chimera.openModels.addRemoveHandler(self._changeModelHandler, None)
        self._addModelHandlerID = chimera.openModels.addAddHandler(self._changeModelHandler, None)
        
    def _changeModelHandler(self,trigger, additional, molList):
        """
        If there is a new model or a model is deleted than update the moleculesList 
        to show the right molecules and the right combination of the different data (id,subid,chain)
        """
        self.loadStruct()
        self.piratesListUpdate()    
        
    def loadStruct(self):
        """
        Creates the dictionary for each molecule that has been loaded into chimera
        Only contains the ptr the name and the id
        """
        self.curModels = chimera.openModels.list(modelTypes=[chimera.Molecule]) #current opened Molecule
        
        self.structures = {}
        for m in self.curModels: #Loop to get all Molecule-names  
            if not self.structures.has_key(m.name):
                self.structures[m.name] = {}
                self.structures[m.name]['ptrModel'] = m
                self.structures[m.name]['id'] = m.id    
        
    def piratesListUpdate(self):
        """
        Updates the list 
        If a new File is loaded this method is called and updates the list with the new molecule
        """
        #self.Chain_Menu.selectitem('*')
        self.structPdbIds = self.structures.keys()
        self.pdbIdMenu.delete(0, END)
        self.varTest = " "
        
        for id_ in self.structPdbIds: #Loop to get all Molecule-names  
            self.varTest += str(id_) + " "
        
        self.pdbIds.set(self.varTest)
        
    def Close(self):
        """
        Closes the GUI
        """       
        if self._closeModelHandlerID != None:
            chimera.openModels.deleteRemoveHandler(self._closeModelHandlerID)
            self._closeModelHandlerID = None
        if self._addModelHandlerID != None:
            chimera.openModels.deleteAddHandler(self._addModelHandlerID)
            self._addModelHandlerID = None
        
        ModelessDialog.Close(self)
        self.destroy()

    def Apply(self): 
        """
        when validation was successful (= data is valid) following happens:
            Creates the result Gui 
            Starts the Thread of the doApply 
            and disable the Apply Button 
        """
        if (validatePirates(self)):
            #possible ERROR is print in validatePriates 
            
            self.buttonWidgets['Apply'].config(state=DISABLED)
            self.r = ResultGui(self.structures)
            t  = Thread(target = self.doApply)
            t.start()
        else:
            tkMessageBox.showerror("ERROR","input is not valid")
 
    def doApply(self):
        '''        
        DataProvider is created and filled
        BioDAppsConnector is created (gets dataProvider to add JobID and send the data to the Server, and gets a callback Method to update the progress)
        bioDAppsC starts Job 
        if something went wrong shows an ErrorMassageBox 
        the result Gui gets the result if the Gui isn't closed
        finally button 'apply' will be activated again (always)
        '''
        #create empty dataProvider and fill it
        dataProvider = dataProviderPirates()
        dataProvider.readGUI(self)
        dataProvider.setPdbs(self.getPDBListPirates())
    
        try:
          
            bioDAppsC = Connector.PiratesConnector(dataProvider, self.r.updateNames)  
            result = bioDAppsC.startPiratesJob()

            try:
                self.r.setRS(result, bioDAppsC)
            except:
                print "you closed the result gui"
            
        except Exception as e:                
            tkMessageBox.showerror("ERROR","Server not available")
            print e
        
        self.buttonWidgets['Apply'].config(state=NORMAL)
        
    def selectAllStructure(self):
        """
        selects all Structures in the pdb Menu 
        """
        self.pdbIdMenu.selection_set(0, END)   
        
    def selectClearStructure(self):
        """
        clear the Selection of the pdb Menu 
        """
        self.pdbIdMenu.selection_clear(0, END)
   
    def selectAll(self):
        """
        select all checkboxes after the button select all was clicked
        """
        self.var1.set(1)
        self.var2.set(1)
        self.var3.set(1)
        self.var4.set(1)
        self.var5.set(1)
        self.var6.set(1)
        self.var7.set(1)
        self.var8.set(1)
    
    def rmSelect(self):
        """
        deselect all checkboxes after the button remove all selections was clicked
        """
        self.var1.set(0)
        self.var2.set(0)
        self.var3.set(0)
        self.var4.set(0)
        self.var5.set(0)
        self.var6.set(0)
        self.var7.set(0)
        self.var8.set(0)
    
    def tgSelect(self):
        """
        Toggles the selection from all checkboxs
        """
        if (self.var1.get() == 0):
            self.var1.set(1)
        else:
            self.var1.set(0)
            
        if (self.var2.get() == 0):
            self.var2.set(1)
        else:
            self.var2.set(0)
            
        if (self.var3.get() == 0):
            self.var3.set(1)
        else:
            self.var3.set(0)
            
        if (self.var4.get() == 0):
            self.var4.set(1)
        else:
            self.var4.set(0)
            
        if (self.var5.get() == 0):
            self.var5.set(1)
        else:
            self.var5.set(0)
            
        if (self.var6.get() == 0):
            self.var6.set(1)
        else:
            self.var6.set(0)
            
        if (self.var7.get() == 0):
            self.var7.set(1)
        else:
            self.var7.set(0)
            
        if (self.var8.get() == 0):
            self.var8.set(1)
        else:
            self.var8.set(0)

    def fillInUI(self, parent):
        """
        Builds the GUI for pirates .
        You can select the loaded molecules of the moleculesList.
        You can also choose between different MStA Methods
        
        bg -> light blue background
        """
        #creat window
        self.parent = parent
        self.frame1=Frame(parent, bg="#d3ebf7")
        self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
        
        #formations box
        self.labelframe1 = LabelFrame(self.frame1, text="Select Structure", bg="#d3ebf7")
        self.labelframe1.pack(expand="yes", padx=0, pady=5)
        
        #labels
        self.frame2 = Frame(self.labelframe1, bg="#d3ebf7")
        self.frame2.grid(row=0, column=0)
        self.label1 = Label(self.frame2, text="Two or more:", bg="#d3ebf7").pack()
        self.selectAllStructureB = Button(self.frame2, text="    Select All Structure   ", command=self.selectAllStructure) 
        self.selectAllStructureB.pack()
        self.selectClearStructureB = Button(self.frame2, text="Clear Structure Selection", command=self.selectClearStructure) 
        self.selectClearStructureB.pack()
        
        #####textbos with Scrollbar
        self.frametext = Frame(self.labelframe1, bg="#d3ebf7")
        self.frametext.grid(row=0, column=1)
        
        self.scrollbar = Scrollbar(self.frametext)
        self.scrollbar.pack(side=RIGHT, fill=Y)
        
        self.pdbIds = StringVar()
        
        #selection list
        self.pdbIdMenu = Listbox(self.frametext, listvariable=self.pdbIds, selectmode=MULTIPLE, width=25, height=7, yscrollcommand=self.scrollbar.set)
        self.pdbIdMenu.pack()
               
        self.loadStruct()
        self.piratesListUpdate()

        self.scrollbar.config(command=self.pdbIdMenu.yview)
        #####
        
        #formations box
        self.labelframe2 = LabelFrame(self.frame1, text="MStA Methods", bg="#d3ebf7")
        self.labelframe2.pack(expand="yes", padx=0, pady=5)
        
        #labels
        self.label2 = Label(self.labelframe2, text="One or more:", bg="#d3ebf7").grid(row=0, column=0)
        
        #variables which know box is checkt
        self.var1 = IntVar()
        self.var2 = IntVar()
        self.var3 = IntVar()
        self.var4 = IntVar()
        self.var5 = IntVar()
        self.var6 = IntVar()
        self.var7 = IntVar()
        self.var8 = IntVar()
        
        #set defalt value
        self.var1.set(1)
        self.var2.set(1)
        self.var3.set(0)
        self.var4.set(1)
        self.var5.set(1)
        self.var6.set(0)
        self.var7.set(0)
        self.var8.set(0)
        
        #checkboxes
        self.check1 = Checkbutton(self.labelframe2, text="MASS", variable=self.var1, bg="#d3ebf7").grid(row=0, column=1, sticky=W)
        self.check2 = Checkbutton(self.labelframe2, text="Matt", variable=self.var2, bg="#d3ebf7").grid(row=1, column=1, sticky=W)
        self.check3 = Checkbutton(self.labelframe2, text="MUSTANG", variable=self.var3, bg="#d3ebf7").grid(row=2, column=1, sticky=W)
        self.check4 = Checkbutton(self.labelframe2, text="MISTRAL", variable=self.var4, bg="#d3ebf7").grid(row=3, column=1, sticky=W)
        self.check5 = Checkbutton(self.labelframe2, text="MAPSCI", variable=self.var5, bg="#d3ebf7").grid(row=4, column=1, sticky=W)
        self.check6 = Checkbutton(self.labelframe2, text="3DCOMB", variable=self.var6, bg="#d3ebf7").grid(row=5, column=1, sticky=W)
        self.check7 = Checkbutton(self.labelframe2, text="Smolign", variable=self.var7, bg="#d3ebf7").grid(row=6, column=1, sticky=W)
        self.check8 = Checkbutton(self.labelframe2, text="msTALI", variable=self.var8, bg="#d3ebf7").grid(row=7, column=1, sticky=W)
        
        #select buttons
        self.selectAllB = Button(self.labelframe2, text="       Select All      ", command=self.selectAll) 
        self.selectAllB.grid(row=2, column=0)
        self.rmSelectB = Button(self.labelframe2, text="Remove Selection", command=self.rmSelect) 
        self.rmSelectB.grid(row=4, column=0)
        self.tgSelectB = Button(self.labelframe2, text=" Toggle Selection ", command=self.tgSelect) 
        self.tgSelectB.grid(row=6, column=0)


    def getPDBListPirates (self):
        '''
        Returns a list of pairs of the selected pdbids and the corresponding pdbstring of the .pdb-file
        '''
        self.selId = self.pdbIdMenu.selection_get() # selected
        idList = str(self.selId).split()
        self.PDBpairs = []
        for iid in idList: 
            model = self.structures[iid]['ptrModel'] #durch ausgewaehltes model ersetzen
            mem = StringIO.StringIO()
            chimera.pdbWrite([model], model.openState.xform, mem)
            mem.seek(0,0) #zurueck auf erste zeile (zeiger war auf der letzten zeile -> deshalb gabs keine ausgabe)
        
            self.pdbstr = mem.getvalue()
            mem.close()
            self.PDBpairs.append((iid,self.pdbstr)) 
        return self.PDBpairs

from chimera import dialogs
dialogs.register(PiratesDialog.name, PiratesDialog)

#---------------------------------------

def getResultById(jobId):
    """
    @param jobId: getting the id for an job 
    this method is called in the BioDapps Gui
    testing if it is finished
    if yes load the result and creat's an result Gui
    """
    structures = loadStructResult();
    bioDAppsC = Connector.PiratesConnector()
    bioDAppsC.setJobId(jobId)
    status = bioDAppsC.getAdapterStatus()['result']
    if "running" in status.values():
        tkMessageBox.showerror("Error", "Job is still running\n Please try later")
        print "job is still running"
    else:
        result = bioDAppsC.getResult()
        r = ResultGui(structures, result, bioDAppsC)  # @UnusedVariable
   
 
def loadStructResult():
    """
    loads the structures from Chimera into the structures Object
    use when bioDApps load the an Job 
    """
    currentModels = chimera.openModels.list(modelTypes=[chimera.Molecule]) #current opened Molecule
    structures = {}
    for m in currentModels: #Loop to get all Molecule-names  
        if not structures.has_key(m.name):
            structures[m.name] = {}
            structures[m.name]['ptrModel'] = m
            structures[m.name]['id'] = m.id  
    return structures 
#---------------------------------------

from Pmw import ComboBox # @UnresolvedImport

class ResultGui (ModelessDialog):
    """
    Generates and fills result GUI 
    You can choose in a List successful MStA Methods and the Consensus
    You can superposition the Structures from the Result in Chimera
    You can get a Sequence Alignment from the Result 
    if the Structures are one Chain Structures The Textalignment is connected to the Chimera Structures and you can Select AAs from the Textalignment
    You can Highlight the Chimera Selection
    You can Select the Core Alignment and Highlight it
    
    inherited from ModelessDialog  
    """
    title="Pirates ResultGui"
    name="Pirates ResultGui"
    buttons=('Close')
    
    def __init__(self, structures, result=None, bioDAppsC=None, **kw):
        '''
        Constructor
        @param structures: are the Structures from Chimera
        @param result: The Consensus result from the Server
        @param bioDAppsC: The Connector to the Server to get the MStA Method results
        
        nameList is a list of the names of the MStA Method   
        label is a dictionary of Labels that shows the progress     
        _master.after is a element/function of the ModelessDialog, is used as invokeLater
        '''        
        #fill in GUI:
        self.methodsMap = {"mass":"MASS", 
                           "matt":"Matt", 
                           "mustang":"MUSTANG", 
                           "mistral":"MISTRAL", 
                           "mapsci":"MAPSCI", 
                           "threedcomb":"3DCOMB", 
                           "smolign":"Smolign", 
                           "mstali":"msTALI", }
        
        self.methodsNameMap = {"MASS":"mass", 
                           "Matt":"matt", 
                           "MUSTANG":"mustang", 
                           "MISTRAL":"mistral", 
                           "MAPSCI":"mapsci", 
                           "3DCOMB":"threedcomb", 
                           "Smolign":"smolign", 
                           "msTALI":"mstali", }
        
        self.result = result
        self.structures = structures
        self.nameList = None
        self.bioDAppsC = bioDAppsC
        if self.result != None:
            self.coreAlignment = getCoreAlignment(self.result[1],self.structures)
        else:
            self.coreAlignment = None
        self.label = {}
        self.close = False
        ModelessDialog.__init__(self, **kw)
        self._master.after(100, self.render)
             
    def Close(self):
        """
        Closes the GUI
        """        
        self.close = True
        if self.result:
            ModelessDialog.Close(self)    
            self.destroy()
             
    def FillMethods(self):
        """
        @return: a list of all names of MStA Method that where successful using the BioDApps Connector 
        """
        fin = self.bioDAppsC.getAdapterStatus()['result']
        methods = ["Consensus"]
        for key in self.methodsMap.keys():
            if key in self.result[2]["result"]:
                if fin[key] == "finished":                 
                    methods.append(self.methodsMap[key]) 
               
        return methods
            
    def updateNames(self, namelist):
        """
        callback Method for the BioDAppas Connector 
        so the Progress can be updated 
        """
        self.nameList = namelist
        #self._master.after(100, self.render)
           
    def setRS(self, result, BioDAppsC):
        """
        @param result: set the result
        @param BioDAppsC: set the BioDAppsC
        """
        self.result = result
        self.bioDAppsC = BioDAppsC
        
    def render(self):
        """
        is called every 1000ms after the ResultGUI is initiated
        if the ResultGUI is not closed or if there is no result 
        you see a the Labels showing you the progress otherwise the GUI will be closed 
        or filled with the given result
        """        
        if self.close:
            ModelessDialog.Close(self)    
            self.destroy()
            return
        
        if self.result != None and self.bioDAppsC != None:
            self.coreAlignment = getCoreAlignment(self.result[1],self.structures)
            
            for element in self.parent.winfo_children() :
                    element.destroy()
            
            self.fillInUI(self.parent)  
        elif self.nameList != None:   
            
            for leb in self.label.keys():
                if leb not in self.nameList.keys():
                    self.label[leb].destroy()
                    del self.label[leb]
                            
            i = 0            
            for name in self.nameList.keys():
                labeltext = name + ":  " + self.nameList[name]
                if name not in self.label.keys():
                    lb = Label(self.labelframe1, text=labeltext, bg="#d3ebf7", font=("Courier", 20))
                    lb.grid(row = i, column = 0)
                    self.label[name] = lb
                else:
                    self.label[name].config(text=labeltext)
             
                i += 1
            self._master.after(1000, self.render)
        else:
            self._master.after(1000, self.render)
        
    def loadMethodsResult(self, entry):
        """
        this Method is called when selecting a new MStA Method at the ComboBox
        load the result for the chosen MStA Method   
        """
        if self.methods_Menu.get() == "Consensus":
            methode = "consensusAlignment"
        else:
            methode  = self.methodsNameMap[self.methods_Menu.get()] + "/result"
        
        self.result = self.bioDAppsC.getResult(alignmentMethod = methode)
        self.coreAlignment = getCoreAlignment(self.result[1],self.structures)
          
    def openTextResult(self):
        """
        uses the MultAlignViewer to show the Text Alignment getting from the Result as Fasta 
        uses the MultAlignViewer.common readFile to get Chimera sequences Object from my Fasta
        uses the MultAlignViewer.MAViewer MAViewer to show the Text Alignment and associate the Chimera Structures and the Text Alignment
        
        """
        
        from MultAlignViewer.common import readFile
        print self.result
        seqs = readFile(self.result[0][1], None)[0]  
        
        from MultAlignViewer.MAViewer import MAViewer
        mav = MAViewer(seqs, title="%s Sequence Alignment" % self.methods_Menu.get(), autoAssociate=False)
        
        ##All models that are loaded in Chimera
        mols = filter(lambda m: isinstance(m, chimera.Molecule),
                        chimera.openModels.list())
        
        
        for aseq in mav.seqs:
            for mol in mols:                    
                if len(mol.sequences()) == 1: ## if the model has only one Chain
                    if mol.id == self.structures[aseq.name]['id']: ## if the id from the mol and from tha aseq from the mav are the same they can be associate
                        mav.associate(mol.sequences()[0], seq=aseq, force=True)
  
    def superposition(self):
        """
        calls the superPos method from piratesExt __init__
        """
        superPos(self.coreAlignment)
    
    def highlight(self, trans):
        """
        @param trans: the amount of the transparency
        gets the Chimera Selection and transparency the everything that's not Selected
        """
        from chimera import selection
        import Midas
         
        selection.invertCurrent(True)
        sel = selection.currentResidues()
        itemsSel = selection.ItemizedSelection()
        itemsSel.add(sel)
        Midas.transparency(trans, sel)
        selection.invertCurrent(True)
        sel = selection.currentResidues()
        itemsSel = selection.ItemizedSelection()
        itemsSel.add(sel)
        Midas.transparency(0, sel)
    
    def selectHighlight(self):
        """
        Highlight the Selection
        """
        self.highlight(self.transparency.get())
    
    def coreHighlight(self):
        """
        Select the Core Alignment
        than Highlight the Selection
        """
        selectCoreAlignment(self.coreAlignment)
        self.highlight(self.transparency.get())
    
    def fillInUI(self, parent):
        """
        if there is a Result shows all the options for the Result
        else give the possibilitie for the Method render to show the Progress  
        """
        self.parent = parent
        if self.result != None:
            self.frame1=Frame(parent, bg="#d3ebf7")
            self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
            
            self.labelframe1 = LabelFrame(self.frame1, text="Methods", bg="#d3ebf7")
            self.labelframe1.pack(expand="yes", padx=0, pady=5)
    
            self.methods_Menu = ComboBox(self.labelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.FillMethods(),selectioncommand=self.loadMethodsResult)
            self.methods_Menu.grid(row=0, column=0, columnspan = 2)
            self.methods_Menu.selectitem(self.methods_Menu.component('scrolledlist').get()[0])
            
            self.sequence_alig = Button(self.labelframe1, text="Sequence Alignment", command=self.openTextResult) 
            self.sequence_alig.grid(row=1, column=0)
            
            self.superpos = Button(self.labelframe1, text="Superposition", command=self.superposition) 
            self.superpos.grid(row=1, column=1)
            
            self.highlight_select = Button(self.labelframe1, text="Highlight Selection", command=self.selectHighlight) 
            self.highlight_select.grid(row=2, column=0)
            
            self.highlight_core = Button(self.labelframe1, text="Highlight Core", command=self.coreHighlight) 
            self.highlight_core.grid(row=2, column=1)
            
            self.transparency = Scale(self.labelframe1, from_=0, to=100, orient=HORIZONTAL, bg="#d3ebf7", length = 200, label="Transparency")
            self.transparency.grid(row=3, column=0, columnspan = 2)
            self.transparency.set(70)
        else:
            self.frame1=Frame(parent, bg="#d3ebf7")
            self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
            
            self.labelframe1 = LabelFrame(self.frame1, text="Please Wait", bg="#d3ebf7")
            self.labelframe1.pack(expand="yes", padx=0, pady=5)

dialogs.register(ResultGui.name, ResultGui)